/**
 * 
 */

package com.qihoo.appstore.updatelib;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.params.HttpClientParams;
import org.apache.http.conn.params.ConnRouteParams;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HTTP;

import android.content.Context;
import android.net.Proxy;
import android.text.TextUtils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.concurrent.TimeoutException;

/**
 * @author zhangyong <zhangyong232@gmail.com>
 */
class HttpHelper {

    static interface ProgressListener {
        void onProgressBegin();

        void onProgressChanged(long progress, long max);

        void onProgressEnd();
    }

    private static void setHttpHeader(Context context, HttpUriRequest request,
            String url) {
        setProxyIfNecessary(context, request, url);
        AndroidHttpClient.modifyRequestToAcceptGzipResponse(request);
    }

    static String getReslUrl(Context context, final String url) throws IOException,
            TimeoutException {
        AndroidHttpClient client = null;
        try {
            client = getHttpClient(context);

            HttpParams params = client.getParams();
            HttpClientParams.setRedirecting(params, false);

            HttpUriRequest request = new HttpGet(url);
            setHttpHeader(context, request, url);
            HttpResponse response = client.execute(request);
            int statusCode = response.getStatusLine().getStatusCode();

            if (HttpStatus.SC_MOVED_TEMPORARILY != statusCode) {
                return url;
            }

            // location:http://m.shouji.360tpcdn.com/131101/23d49779d04528e8bf14af34e07a14ad/com.qihoo.appstore_109020101.apk
            Header header = response.getFirstHeader("location");
            if (header != null) {
                return header.getValue();
            } else {
                return url;
            }
        } finally {
            try {
                if (client != null) {
                    client.close();
                }
            } catch (Throwable e) {
            }
        }
    }

    /** 执行HTTP請求，返回http返回的字符串 **/
    public static String get(Context context, final String url)
            throws ClientProtocolException, IOException {
        final AndroidHttpClient client = getHttpClient(context);
        try {
            HttpUriRequest request = new HttpGet(url);
            setHttpHeader(context, request, url);
            HttpResponse response = client.execute(request);
            int statusCode = response.getStatusLine().getStatusCode();
            if (HttpStatus.SC_OK != statusCode) {
                throw new ClientProtocolException(
                        "The status code of http is not SC_OK(200):"
                                + statusCode + ",\r\n\turl=" + url);
            }
            HttpEntity entity = response.getEntity();

            if (entity == null) {
                throw new ClientProtocolException("No content be send");
            }
            return EntityUtils.toString(response.getEntity(),
                    HTTP.DEFAULT_CONTENT_CHARSET);
        } finally {
            try {
                client.close();
            } catch (Throwable e) {
            }
        }
    }

    /** 执行HTTP请求，将内容保存到一个文件，如果文件存在，则断点续传 **/
    public static String get(Context context, final String url,
            final File filename, ProgressListener listener) throws ClientProtocolException,
            IOException {
        final AndroidHttpClient client = getHttpClient(context);
        try {

            if (listener != null) {
                listener.onProgressBegin();
            }

            HttpGet request = new HttpGet(url);
            setHttpHeader(context, request, url);
            if (filename.exists() && filename.isFile() && filename.length() > 0) {
                request.addHeader("RANGE", "bytes=" + filename.length() + "-");
            }

            HttpResponse response = client.execute(request);
            int statusCode = response.getStatusLine().getStatusCode();

            if (HttpStatus.SC_OK != statusCode && HttpStatus.SC_PARTIAL_CONTENT != statusCode) {
                throw new ClientProtocolException(
                        "The status code of http is not SC_OK(200) or PARTIAL_CONTENT(206):"
                                + statusCode + ",\r\n\turl=" + url);
            }

            if (HttpStatus.SC_OK == statusCode) {
                filename.delete();
            }

            HttpEntity entity = response.getEntity();

            if (entity == null) {
                throw new ClientProtocolException("No content be send");
            }

            BufferedInputStream in = null;
            BufferedOutputStream out = null;
            try {
                long hasReaded = filename.exists() ? filename.length() : 0;
                long totle = hasReaded + getContentLength(response);

                if (listener != null) {
                    listener.onProgressChanged(hasReaded, totle);
                }

                out = new BufferedOutputStream(new FileOutputStream(filename, true));
                in = new BufferedInputStream(
                        AndroidHttpClient.getUngzippedContent(response
                                .getEntity()));
                byte[] buffer = new byte[8192];
                int readed = 0;

                while ((readed = in.read(buffer)) != -1) {
                    out.write(buffer, 0, readed);
                    hasReaded += readed;
                    if (listener != null) {
                        listener.onProgressChanged(hasReaded, totle);
                    }
                }
                out.flush();
                return filename.getPath();
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (Exception e) {
                    }
                }
                if (out != null) {
                    try {
                        out.close();
                    } catch (Exception e) {
                    }
                }
            }
        } finally {
            try {
                client.close();
            } catch (Throwable e) {
            }
            if (listener != null) {
                listener.onProgressEnd();
            }
        }
    }

    /** 执行HTTP請求，自己处理响应 **/
    public static <T> T get(Context context, String url,
            ResponseHandler<? extends T> handler)
            throws ClientProtocolException, IOException {
        final AndroidHttpClient client = getHttpClient(context);
        try {
            HttpUriRequest request = new HttpGet(url);
            setHttpHeader(context, request, url);
            return client.execute(request, handler);
        } finally {
            client.close();
        }
    }

    private static long getContentLength(HttpResponse response) {
        final HttpEntity entity = response.getEntity();
        if (entity == null) {
            return -1;
        }
        return entity.getContentLength();
    }

    private static AndroidHttpClient getHttpClient(Context context) {
        AndroidHttpClient c = AndroidHttpClient.newInstance(
                "360Download", context);
        HttpParams params = c.getParams();
        HttpClientParams.setRedirecting(params, true);
        return c;
    }

    private static void setProxyIfNecessary(Context context,
            HttpUriRequest request, String url) {
        // Set or unset proxy, which may have changed since last GET request.
        // setDefaultProxy() supports null as proxy parameter.
        ConnRouteParams.setDefaultProxy(request.getParams(),
                getProxy(context, url));
    }

    private static HttpHost getProxy(Context context, String url) {
        @SuppressWarnings("deprecation")
        final String host = Proxy.getDefaultHost();
        @SuppressWarnings("deprecation")
        final int port = Proxy.getDefaultPort();
        if (TextUtils.isEmpty(host) || port <= 0) {
            return null;
        }
        return new HttpHost(host, port, "http");
    }
}
