/**
 * 
 */

package com.qihoo.appstore.updatelib;

import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.support.v4.app.NotificationCompat;
import android.text.TextUtils;
import android.text.format.Formatter;
import android.util.Log;

import java.io.File;

/**
 * @author zhangyong6
 */
class NotificationHelper {

    private static final String ACTION_NULL = "com.qihoo.update.action_NULL";
    private static final int NOTIFY_ID = 0x19880515;

    private static final String TAG = "NotificationHelper";

    static void notifyMessage(Context context, String title,
            String message, PendingIntent intent) {
        NotificationManager nm = (NotificationManager) context
                .getSystemService(Context.NOTIFICATION_SERVICE);

        NotificationCompat.Builder builder = new NotificationCompat.Builder(context);
        builder.setWhen(System.currentTimeMillis()).setAutoCancel(true)
                .setSmallIcon(context.getApplicationInfo().icon);

        if (!TextUtils.isEmpty(title)) {
            builder.setContentTitle(title);
        }

        if (!TextUtils.isEmpty(message)) {
            builder.setContentText(message);
        }
        builder.setContentIntent(intent);

        nm.cancel(NOTIFY_ID);
        nm.notify(NOTIFY_ID, builder.build());
    }

    static void notifyProgressMessage(Context context, String title,
            String message, PendingIntent intent, int progress, int max, boolean indeterminate) {
        if (Build.VERSION.SDK_INT < 16) {
            notifyProgressMessageApi16(context, title, message, intent, progress, max,
                    indeterminate);
        } else {
            NotificationManager nm = (NotificationManager) context
                    .getSystemService(Context.NOTIFICATION_SERVICE);
            NotificationCompat.Builder builder = new NotificationCompat.Builder(context);
            builder.setWhen(System.currentTimeMillis()).setAutoCancel(false)
                    .setSmallIcon(context.getApplicationInfo().icon).setOngoing(true)
                    .setOnlyAlertOnce(true);

            if (!TextUtils.isEmpty(title)) {
                builder.setContentTitle(title);
            }

            if (!TextUtils.isEmpty(message)) {
                builder.setContentText(message);
            }
            if (!indeterminate && max > 0) {
                builder.setContentInfo(String
                        .format("%.0f%%", (float) progress / (float) max * 100));
            }
            builder.setProgress(max, progress, indeterminate);
            builder.setContentIntent(intent);
            nm.notify(NOTIFY_ID, builder.build());
        }
    }

    static void notifyProgressMessageApi16(Context context, String title,
            String message, PendingIntent intent, int progress, int max, boolean indeterminate) {
        NotificationManager nm = (NotificationManager) context
                .getSystemService(Context.NOTIFICATION_SERVICE);
        NotificationCompat.Builder builder = new NotificationCompat.Builder(context);
        builder/* .setWhen(System.currentTimeMillis()) */.setAutoCancel(false)
                .setSmallIcon(context.getApplicationInfo().icon).setOngoing(true)
                .setOnlyAlertOnce(true);

        if (!TextUtils.isEmpty(title)) {
            builder.setContentTitle(title);
        }

        if (!TextUtils.isEmpty(message)) {
            if (!indeterminate && max > 0) {
                message += " " + String
                        .format("%.0f%%", (float) progress / (float) max * 100);
            }
            builder.setContentText(message);
        }
        builder.setContentIntent(intent);
        nm.notify(NOTIFY_ID, builder.build());
    }

    static void notifyDownloadComplete(Context context, String apkTitle, File file) {
        try {
            if (file.exists()) {
                Intent myintent = new Intent(Intent.ACTION_VIEW);
                myintent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                Uri apkUri = Uri.fromFile(new File(file.getPath()));
                myintent.setDataAndType(apkUri, "application/vnd.android.package-archive");

                PendingIntent pendingIntent = PendingIntent.getActivity(context,
                        (int) System.currentTimeMillis(), myintent,
                        PendingIntent.FLAG_UPDATE_CURRENT);
                String title = RHelper.getString(context,
                        "appstore_notification_title_apk_download_ok");

                String message = RHelper.getString(context,
                        "appstore_notification_message_apk_download_ok",
                        TextUtils.isEmpty(apkTitle) ? "" : apkTitle);
                NotificationHelper.notifyMessage(context, title, message,
                        pendingIntent);
            } else {
                String title = RHelper.getString(context,
                        "appstore_notification_title_apk_download_fail");
                String message = RHelper.getString(context,
                        "appstore_notification_message_apk_download_fail",
                        TextUtils.isEmpty(apkTitle) ? "" : apkTitle);
                PendingIntent pendingIntent = PendingIntent.getBroadcast(context,
                        (int) System.currentTimeMillis(), new Intent(ACTION_NULL),
                        PendingIntent.FLAG_UPDATE_CURRENT);
                NotificationHelper.notifyMessage(context, title, message,
                        pendingIntent);
            }
        } catch (Throwable e) {
            if (UpdateManager.isDebug()) {
                Log.e(TAG, "notifyDownloadProgress", e);
            }
        }
    }

    static void notifyDownloadProgress(Context context, final String apkTitle, long progress,
            long max) {
        try {
            String title = RHelper.getString(context,
                    "appstore_notification_title_apk_downloading");
            String message = RHelper.getString(context,
                    "appstore_notification_message_apk_downloading",
                    TextUtils.isEmpty(apkTitle) ? "" : apkTitle);
            PendingIntent pendingIntent = PendingIntent.getBroadcast(
                    context,
                    (int) System.currentTimeMillis(), new Intent(ACTION_NULL),
                    PendingIntent.FLAG_UPDATE_CURRENT);

            if (max <= 0 || progress < 0) {
                NotificationHelper.notifyProgressMessage(context,
                        title, message,
                        pendingIntent,
                        0, 100, true);
            } else {
                NotificationHelper.notifyProgressMessage(context,
                        title, message,
                        pendingIntent,
                        (int) (100f * (float) progress / max), 100, false);
            }
        } catch (Throwable e) {
            if (UpdateManager.isDebug()) {
                Log.e(TAG, "notifyDownloadProgress", e);
            }
        }
    }

    public static void notifyNewUpdate(Context context, AppInfo info) {
        if (!info.isNewVersion) {
            return;
        }

        try {
            PackageManager pm = context.getPackageManager();
            CharSequence label = pm.getApplicationLabel(pm.getApplicationInfo(info.packageName, 0));
            label = TextUtils.isEmpty(label) ? "" : label;
            String title = RHelper.getString(context,
                    "appstore_notification_title_found_new_version", label);
            String message;
            if (info.isPatchUpdate) {
                message = RHelper.getString(context,
                        "appstore_notification_message_found_new_version_patch",
                        info.versioName,
                        Formatter.formatFileSize(context, info.size - info.patchUpdateSize));
            } else {
                message = RHelper.getString(context,
                        "appstore_notification_message_found_new_version_speed",
                        info.versioName, Formatter.formatFileSize(context, info.size));
            }

            Intent checkUpdateAcitivtyIntent = new Intent(context,
                    CheckUpdateAcitivty.class);
            checkUpdateAcitivtyIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            checkUpdateAcitivtyIntent.putExtra(CheckUpdateService.EXTRA_UPDATE_INFO, info);

            PendingIntent pendingIntent = PendingIntent.getActivity(context,
                    (int) System.currentTimeMillis(), checkUpdateAcitivtyIntent,
                    PendingIntent.FLAG_UPDATE_CURRENT);
            NotificationHelper.notifyMessage(context, title, message,
                    pendingIntent);
        } catch (Throwable e) {
            if (UpdateManager.isDebug()) {
                Log.e(TAG, "notifyDownloadProgress", e);
            }
        }
    }
}
