/**
 * 
 */

package com.qihoo.appstore.updatelib;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.Log;
import android.webkit.URLUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.security.MessageDigest;

/**
 * @author zhangyong6
 */
class Utils {
    private static final String TAG = "Utils";

    private static final String QIHOO_UPDATE_CHANNEL = "QIHOO_UPDATE_CHANNEL";

    static String getAppStoreDownloadUrl(Context context) {
        return "http://openbox.mobilem.360.cn/channel/getUrl?src=" + getChannel(context)
                + "&app=360box";
    }

    private static String sChannelCache = null;

    static String getChannel(final Context context) {
        try {
            if (sChannelCache == null) {
                String packageName = context.getPackageName();
                PackageManager packageManager = context.getPackageManager();
                ApplicationInfo info = packageManager.getApplicationInfo(packageName,
                        PackageManager.GET_META_DATA);
                Object data = info.metaData != null ? info.metaData
                        .get(QIHOO_UPDATE_CHANNEL)
                        : null;

                sChannelCache = data != null ? data.toString() : null;
                if (TextUtils.isEmpty(sChannelCache) && info.metaData != null
                        && info.metaData.containsKey(QIHOO_UPDATE_CHANNEL)) {
                    sChannelCache = String.valueOf(info.metaData.get(QIHOO_UPDATE_CHANNEL));
                }
            }
            return sChannelCache;
        } catch (NameNotFoundException e) {
            return null;
        }
    }

    static void logForIntent(final Context context) {
        new Thread() {
            @Override
            public void run() {
                try {
                    StringBuilder sb = new StringBuilder(
                            "http://openbox.mobilem.360.cn/AppStore/info?");
                    sb.append("m=").append(Utils.getIMEI(context));
                    sb.append("&m2=").append(Utils.getIMEI2(context));
                    sb.append("&chpart=").append(Utils.getChannel(context));
                    sb.append("&func=2&param=1");
                    HttpHelper.get(context, sb.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.start();

    }

    private static final String START_ACTIVITY_INDEX = "start_activity_index";
    private static final String START_ACTIVITY_TO_APP_INFO_AUTO_DOWNLOAD_KEY = "auto_download";
    private static final int APP_STORE_INTENT_START_APP_INFO_INDEX = 30;
    static final String APPSTORE_PKGNAME = "com.qihoo.appstore";
    private static final String START_ACTIVITY_SOURCE_STYPE_TYPE = "from_out_side_start_type";
    private static final String START_ACTIVITY_FROM_OUTSIDE_FROM = "from_out_side";

    // appstore low version intent to appinfo page
    private static final String LAUNCH_APPSTORE_KEY = "activityKey";
    private static final String LAUNCH_APPSTORE_APP_INFO_ACTIVITY = "appInfo";
    private static final String LAUNCH_APPSTORE_PACKAGE_NAME = "search_content";

    private static boolean isLowVersion(Context context) {
        boolean result = true;
        PackageManager pm = context.getPackageManager();
        try {
            PackageInfo info = pm.getPackageInfo(APPSTORE_PKGNAME, 0);
            if (info.versionCode >= 199801) {
                result = false;
            }
        } catch (NameNotFoundException e) {
            e.printStackTrace();
        }
        return result;
    }

    static int startAppStoreToUpdate(Context context, String pkgName,
            boolean autoDownload) {

        try {
            if (TextUtils.isEmpty(pkgName)) {
                return -1;
            }

            if (!isPackageInstalled(context, APPSTORE_PKGNAME)) {
                return -2;
            }

            Utils.logForIntent(context);

            Intent intent = context.getPackageManager().getLaunchIntentForPackage(APPSTORE_PKGNAME);
            intent.putExtra(START_ACTIVITY_SOURCE_STYPE_TYPE, 1302);// 这个值必需修改，由助手分配，具体值可以问手机助手工作人员
            intent.putExtra("startype", 1302); // 这个值必需修改，由助手分配，具体值可以问手机助手工作人员
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            if (isLowVersion(context)) {
                // 旧版本提供自动下载功能
                intent.putExtra(LAUNCH_APPSTORE_KEY, LAUNCH_APPSTORE_APP_INFO_ACTIVITY);
                intent.putExtra(LAUNCH_APPSTORE_PACKAGE_NAME, pkgName);
                intent.putExtra("exit_directly", true);
                context.startActivity(intent);
                return 0;
            } else {
                intent.putExtra(START_ACTIVITY_INDEX, APP_STORE_INTENT_START_APP_INFO_INDEX);
                intent.putExtra(START_ACTIVITY_FROM_OUTSIDE_FROM, pkgName);
                intent.putExtra("pkg_name", pkgName);
                intent.putExtra(START_ACTIVITY_TO_APP_INFO_AUTO_DOWNLOAD_KEY, autoDownload);
                intent.putExtra("exit_directly", true);
                context.startActivity(intent);
                return 0;
            }
        } catch (Exception e) {
            if (UpdateManager.isDebug()) {
                Log.e(TAG, "startAppStoreToUpdate fail", e);
            }
            return -3;
        }
    }

    static boolean isPackageInstalled(Context context, String packageName) {
        try {
            return context.getPackageManager().getPackageInfo(packageName, 0) != null;
        } catch (Exception e) {
            return false;
        }
    }

    static String getCheckUpdateUrl(Context context, String packageName,
            String channel) {
        String md5 = Utils.md5Package(context, packageName);
        int versionCode = Utils.getVersionCode(context, packageName);

        StringBuilder sb = new StringBuilder("http://openbox.mobilem.360.cn/index/upgrade?");
        sb.append("package=").append(packageName);
        sb.append("&version=").append(versionCode);
        sb.append("&os=").append(Build.VERSION.SDK_INT);
        sb.append("&md5=").append(md5);
        sb.append("&m=").append(Utils.getIMEI(context));
        sb.append("&m2=").append(Utils.getIMEI2(context));
        sb.append("&chpart=").append(channel);
        sb.append("&sdk_version=").append(UpdateManager.SDK_VERSION);
        return sb.toString();
    }

    static void installApk(Context context, String path) {
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        Uri apkUri = Uri.fromFile(new File(path));
        intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
        context.startActivity(intent);
    }

    static String md5Package(Context context, String packageName) {
        try {
            String file = context.getPackageManager().getApplicationInfo(packageName, 0).publicSourceDir;
            return md5(new File(file));
        } catch (NameNotFoundException e) {
            return null;
        }
    }

    static String md5(File file) {
        InputStream fis = null;
        byte[] buffer = new byte[1024];
        int numRead = 0;
        MessageDigest md5;

        try {
            fis = new FileInputStream(file);
            md5 = MessageDigest.getInstance("MD5");
            while ((numRead = fis.read(buffer)) > 0) {
                md5.update(buffer, 0, numRead);
            }

            return toHex(md5.digest());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return null;
    }

    static String md5(String string) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(string.getBytes());
            byte b[] = md.digest();
            int i;
            StringBuffer buf = new StringBuffer("");
            for (int offset = 0; offset < b.length; offset++) {
                i = b[offset];
                if (i < 0)
                    i += 256;
                if (i < 16)
                    buf.append("0");
                buf.append(Integer.toHexString(i));
            }
            return buf.toString();
        } catch (Exception e) {
            return null;
        }
    }

    private static String toHex(byte[] b) {
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < b.length; i++) {
            int v = b[i];
            builder.append(HEX[(0xF0 & v) >> 4]);
            builder.append(HEX[0x0F & v]);
        }
        return builder.toString();
    }

    private static final char[] HEX = {
            '0', '1', '2', '3', '4', '5', '6', '7',
            '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
    };

    private static String sImei2;
    private static String sImei = "360imei";

    static String getIMEI(Context c) {
        try {
            if ("360imei".equals(sImei)) {
                TelephonyManager telephonyManager = (TelephonyManager) c
                        .getSystemService(Context.TELEPHONY_SERVICE);
                sImei = telephonyManager.getDeviceId();
                if (sImei == null) {
                    sImei = "360_DEFAULT_IMEI";
                }
            }
        } catch (Exception e) {
        }
        return sImei;
    }

    // 提供一个新的设备硬件编号，Daemon和Baohe统一用这个接口获取硬件编号
    static String getIMEI2(Context context) {
        sImei2 = PreferenceManager.getDefaultSharedPreferences(context).getString("app_store_imei",
                "");
        if (!TextUtils.isEmpty(sImei2)) {
            return sImei2;
        }

        TelephonyManager tm = (TelephonyManager) context
                .getSystemService(Context.TELEPHONY_SERVICE);
        String imei = tm.getDeviceId();
        String AndroidID = android.provider.Settings.System.getString(
                context.getContentResolver(), "android_id");
        String serialNo = getDeviceSerial();
        sImei2 = md5(imei + AndroidID + serialNo);
        PreferenceManager.getDefaultSharedPreferences(context).edit()
                .putString("app_store_imei", sImei2).commit();
        return sImei2;
    }

    private static String getDeviceSerial() {
        String serial = "";
        try {
            Class<?> c = Class.forName("android.os.SystemProperties");
            Method get = c.getMethod("get", String.class);
            serial = (String) get.invoke(c, "ro.serialno");
        } catch (Exception ignored) {
        }
        return serial;
    }

    static int getVersionCode(Context context, String packageName) {
        try {
            return context.getPackageManager().getPackageInfo(
                    packageName, 0).versionCode;
        } catch (NameNotFoundException e) {
            e.printStackTrace();
        }
        return -1;
    }

    private static File getExternalCacheDir(Context context) {
        if (Build.VERSION.SDK_INT >= 8) {
            return context.getExternalCacheDir();
        } else {
            return new File(Environment.getExternalStorageDirectory(), "Android/data/"
                    + context.getPackageName() + "/cache");
        }
    }

    private static File getInternalCacheDir(Context context) {
        return context.getCacheDir();
    }

    static String getUpdateFile(Context context, String url, String packageName) {
        String fileName = URLUtil.guessFileName(url, null, null);
        final File cacheDir;
        if (Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
            cacheDir = new File(getExternalCacheDir(context), "QihooUpdate");
        } else {
            cacheDir = new File(getInternalCacheDir(context), "QihooUpdate");
        }
        File file = new File(cacheDir, String.format("%s_%s", packageName, fileName));
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        return file.getPath();
    }

    static String getUpdateTempFile(Context context, String url) {

        final File cacheDir;
        if (Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
            cacheDir = new File(getExternalCacheDir(context), "QihooUpdate");
        } else {
            cacheDir = new File(getInternalCacheDir(context), "QihooUpdate");
        }
        File file = new File(cacheDir, String.format("%s.qihootmp", Utils.md5(url)));
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        return file.getPath();
    }
}
